@@ -33,10 +33,28 @@
 #include <gtest/gtest.h>
 #include <xorg/gtest/xorg-gtest.h>
 #include <X11/Xlib.h>
+#include <stdexcept>
 
 namespace xorg {
 namespace testing {
 
+/**
+ * @class XIOError
+ *
+ * Exception thrown if the display connection encounters an IO error and
+ * calls the XIOErrorHandler function.
+ *
+ * This exception requires an XIOErrorHandler to be registered.
+ * XServer::Start() will register this error handler. For tests that do not
+ * use the provided XServer object, call XServer::RegisterXIOErrorHandler()
+ * instead.
+ */
+class XIOError : public std::runtime_error {
+public:
+  /** Create a new XIOError with the given message */
+  XIOError(const std::string& msg) : std::runtime_error(msg) {}
+};
+
 /**
  * @class XServer xorg-gtest-xserver.h xorg/gtest/xorg-gtest-xserver.h
  *
@@ -55,6 +73,9 @@ namespace testing {
  *     std::cerr << "Problem killing server" << std::endl;
  * }
  * @endcode
+ *
+ * Once a XServer is started, a default XIOErrorHandler is installed and
+ * subsequent IO errors on the display connection will throw an XIOError.
  */
 class XServer : public xorg::testing::Process {
   public:
@@ -67,7 +88,7 @@ class XServer : public xorg::testing::Process {
      *
      * @param [in] program Path to the XServer binary
      */
-    void Start(const std::string &program = "");
+    virtual void Start(const std::string &program = "");
 
     /**
      * Terminates this server process. Will signal the server to terminate
@@ -80,7 +101,7 @@ class XServer : public xorg::testing::Process {
      * @returns true if termination succeeded and, if a timout is given, the
      *          process shut down within that timeout. false otherwise.
      */
-    virtual bool Terminate(unsigned int timeout = 0);
+    virtual bool Terminate(unsigned int timeout = 2000);
 
     /**
      * Kills the server. With a vengeance.
@@ -92,7 +113,7 @@ class XServer : public xorg::testing::Process {
      * @returns true if kill succeeded and, if a timout is given, the
      *          process shut down within that timeout. false otherwise.
      */
-    virtual bool Kill(unsigned int timeout = 0);
+    virtual bool Kill(unsigned int timeout = 2000);
 
     /**
      * Remove the log file used by this server. By default, this function
@@ -109,7 +130,7 @@ class XServer : public xorg::testing::Process {
     /**
      * Waits until this server is ready to take connections.
      */
-    void WaitForConnections(void);
+    _X_DEPRECATED void WaitForConnections(void);
 
     /**
      * Set the display number for this server. This number must be set
@@ -180,7 +201,17 @@ class XServer : public xorg::testing::Process {
      * @param [in] key Commandline option
      * @param [in] value Option value (if any)
      */
-    void SetOption(const std::string &key, const std::string &value);
+    void SetOption(const std::string &key, const std::string &value = "");
+
+    /**
+     * Remove a previously set option.
+     *
+     * If an option was set through SetOption(), remove the option again. If
+     * the specified option has never been set, do nothing.
+     *
+     * @param [in] option Commandline option to remove
+     */
+    void RemoveOption(const std::string &option);
 
     /**
      * Wait for a specific device to be added to the server.
@@ -219,7 +250,33 @@ class XServer : public xorg::testing::Process {
      *
      * @return Whether an event is available
      */
-    static bool WaitForEventOfType(::Display *display, int type, int extension, int evtype, time_t timeout = 1000);
+    static bool WaitForEventOfType(::Display *display, int type, int extension = -1, int evtype = -1, time_t timeout = 1000);
+
+    /**
+     * Install a default XIOErrorHandler. That error handler will throw an
+     * xorg::testing::XIOError when encountered.
+     *
+     * This function is called automatically by XServer::Start(). Usually,
+     * you will not need to call this function unless your test does not
+     * instantiate and Start() an XServer object.
+     *
+     * This function will only install a new error handler if the currently
+     * installed XIOErrorHandler is not the default handler used by Xlib.
+     */
+    static void RegisterXIOErrorHandler();
+
+    /**
+     * Install a default XErrorHandler. That error handler will cause a test
+     * failure if called.
+     *
+     * This function is called automatically by XServer::Start(). Usually,
+     * you will not need to call this function unless your test does not
+     * instantiate and Start() an XServer object.
+     *
+     * This function will only install a new error handler if the currently
+     * installed XErrorHandler is not the default handler used by Xlib.
+     */
+    static void RegisterXErrorHandler();
 
   private:
     struct Private;
